# $Id: google-stations.py,v 1.27 2003/05/26 01:23:56 jylefort Exp $
#
# This file is part of streamtuner-python.
#
# Copyright (c) 2003, Jean-Yves Lefort
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of Jean-Yves Lefort nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import re, os, G, ST

FIELD_TITLE 		= 0
FIELD_DESCRIPTION	= 1
FIELD_URL 		= 2

GOOGLE_ROOT		= "http://directory.google.com"
CATEGORIES_URL_POSTFIX	= "/Top/Arts/Music/Sound_Files/MP3/Streaming/Stations/"

re_category	= re.compile('^<a href="(.*)">(<b>)?(.*?)(</b>)?</a>&nbsp;')
re_stream	= re.compile('^<td><font face="arial,sans-serif"><a href="(.*)">(.*)</a>')
re_description	= re.compile('^<br><font size=-1> (.*?)</font>')

class ReloadInfo:
    def __init__ (self, streams):
        self.stream = None
        self.streams = streams

def to_utf8 (str):
    # FIXME: we assume that Google's encoding is ISO8859-1, but we
    # should get it from the Content-Type header (that will require
    # modifying the streamtuner core transfer functions).
    return unicode(str, "iso-8859-1").encode("utf8")
    
def reload_cb (category, categories, streams, data):
    global GOOGLE_ROOT
    
    if not ST.Transfer.get_lines(GOOGLE_ROOT + CATEGORIES_URL_POSTFIX,
                                 categories_line_cb,
                                 categories):
        return None                     # transfer aborted
    
    if category.url_postfix:
        info = ReloadInfo(streams)
        if not ST.Transfer.get_lines(GOOGLE_ROOT + category.url_postfix,
                                     streams_line_cb,
                                     info):
            return None                 # transfer aborted

    return 1

def categories_line_cb (line, categories):
    global re_category
    global CATEGORIES_URL_POSTFIX
    
    match = re_category.match(line)
    if match:
        category = ST.Category()

        category.name = to_utf8(match.group(3))
        category.label = to_utf8(match.group(3))

        href = to_utf8(match.group(1))

        if href[0] == "/":
            category.url_postfix = href
        else:
            category.url_postfix = CATEGORIES_URL_POSTFIX + href
            
        categories.append(G.Node(category))

def streams_line_cb (line, info):
    match = re_stream.match(line)
    if match:
        if info.stream:
            ST.notice("Google Stations: found incomplete stream")
        else:
            info.stream = ST.Stream()
            info.stream.name = to_utf8(match.group(1))	# we use url as name
            info.stream.set_field(FIELD_URL, to_utf8(match.group(1)))
            info.stream.set_field(FIELD_TITLE, to_utf8(match.group(2)))
    else:
        match = re_description.match(line)
        if match:
            if not info.stream:
                ST.notice("Google Stations: found misplaced description")
            else:
                info.stream.set_field(FIELD_DESCRIPTION, to_utf8(match.group(1)))
                info.streams.append(info.stream)
                info.stream = None
        
def browse_cb (stream, data):
    ST.Programs.run("Browse a website", stream.get_field(FIELD_URL))
    return 1

def init_handler ():
    stock_categories = G.Node()
    
    category = ST.Category()
    category.name = "__main"
    category.label = "Internet radios"
    category.url_postfix = "/Top/Arts/Radio/Internet/"
    
    stock_categories.append(G.Node(category))
     
    handler = ST.Handler("google-stations.py")
    handler.label = "Google Stations"
    handler.copyright = "Copyright (c) 2003 Jean-Yves Lefort"
    handler.stock_categories = stock_categories
    
    icon = ST.find_icon("google-stations.png")
    if icon:
        handler.icon = icon
    else:
        ST.notice("Google Stations: unable to find icon file")

    handler.add_field(ST.HandlerField(FIELD_TITLE, "Title", G.TYPE_STRING, 1))
    handler.add_field(ST.HandlerField(FIELD_DESCRIPTION, "Description", G.TYPE_STRING, 1))
    handler.add_field(ST.HandlerField(FIELD_URL, "URL", G.TYPE_STRING, 0))
    
    handler.bind(ST.Handler.EVENT_RELOAD, reload_cb)
    handler.bind(ST.Handler.EVENT_STREAM_BROWSE, browse_cb)
    
    ST.Handlers.add(handler)

def init ():
    if not ST.check_api_version(0, 1):
        raise RuntimeError, "API version mismatch"

    init_handler()
    ST.Programs.register("Browse a website", 'galeon "%"')

init()
